@@ -1,8 +1,8 @@
 /*
  * LUKS - Linux Unified Key Setup v2, PBKDF2 digest handler (LUKS1 compatible)
  *
- * Copyright (C) 2015-2018, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2015-2018, Milan Broz. All rights reserved.
+ * Copyright (C) 2015-2023 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2015-2023 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,10 +33,10 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 	char checkHashBuf[64];
 	json_object *jobj_digest, *jobj1;
 	const char *hashSpec;
-	char *mkDigest = NULL, mkDigestSalt[LUKS_SALTSIZE];
+	char *mkDigest = NULL, *mkDigestSalt = NULL;
 	unsigned int mkDigestIterations;
 	size_t len;
-	int r;
+	int r = -EINVAL;
 
 	/* This can be done only for internally linked digests */
 	jobj_digest = LUKS2_get_digest_jobj(crypt_get_hdr(cd, CRYPT_LUKS2), digest);
@@ -53,25 +53,23 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 
 	if (!json_object_object_get_ex(jobj_digest, "salt", &jobj1))
 		return -EINVAL;
-	len = sizeof(mkDigestSalt);
-	if (!base64_decode(json_object_get_string(jobj1),
-			   json_object_get_string_len(jobj1), mkDigestSalt, &len))
-		return -EINVAL;
+	r = crypt_base64_decode(&mkDigestSalt, &len, json_object_get_string(jobj1),
+				json_object_get_string_len(jobj1));
+	if (r < 0)
+		goto out;
 	if (len != LUKS_SALTSIZE)
-		return -EINVAL;
+		goto out;
 
 	if (!json_object_object_get_ex(jobj_digest, "digest", &jobj1))
-		return -EINVAL;
-	len = 0;
-	if (!base64_decode_alloc(json_object_get_string(jobj1),
-			   json_object_get_string_len(jobj1), &mkDigest, &len))
-		return -EINVAL;
+		goto out;
+	r = crypt_base64_decode(&mkDigest, &len, json_object_get_string(jobj1),
+				json_object_get_string_len(jobj1));
+	if (r < 0)
+		goto out;
 	if (len < LUKS_DIGESTSIZE ||
 	    len > sizeof(checkHashBuf) ||
-	    (len != LUKS_DIGESTSIZE && len != (size_t)crypt_hash_size(hashSpec))) {
-		free(mkDigest);
-		return -EINVAL;
-	}
+	    (len != LUKS_DIGESTSIZE && len != (size_t)crypt_hash_size(hashSpec)))
+		goto out;
 
 	r = -EPERM;
 	if (crypt_pbkdf(CRYPT_KDF_PBKDF2, hashSpec, volume_key, volume_key_len,
@@ -80,11 +78,12 @@ static int PBKDF2_digest_verify(struct crypt_device *cd,
 			mkDigestIterations, 0, 0) < 0) {
 		r = -EINVAL;
 	} else {
-		if (memcmp(checkHashBuf, mkDigest, len) == 0)
+		if (crypt_backend_memeq(checkHashBuf, mkDigest, len) == 0)
 			r = 0;
 	}
-
+out:
 	free(mkDigest);
+	free(mkDigestSalt);
 	return r;
 }
 
@@ -94,18 +93,25 @@ static int PBKDF2_digest_store(struct crypt_device *cd,
 	size_t volume_key_len)
 {
 	json_object *jobj_digest, *jobj_digests;
-	char salt[LUKS_SALTSIZE], digest_raw[128], num[16];
+	char salt[LUKS_SALTSIZE], digest_raw[128];
 	int hmac_size, r;
 	char *base64_str;
 	struct luks2_hdr *hdr;
 	struct crypt_pbkdf_limits pbkdf_limits;
+	const struct crypt_pbkdf_type *pbkdf_cd;
 	struct crypt_pbkdf_type pbkdf = {
 		.type = CRYPT_KDF_PBKDF2,
-		.hash = "sha256",
 		.time_ms = LUKS_MKD_ITERATIONS_MS,
 	};
 
-	log_dbg("Setting PBKDF2 type key digest %d.", digest);
+	/* Inherit hash from PBKDF setting */
+	pbkdf_cd = crypt_get_pbkdf_type(cd);
+	if (pbkdf_cd)
+		pbkdf.hash = pbkdf_cd->hash;
+	if (!pbkdf.hash)
+		pbkdf.hash = DEFAULT_LUKS1_HASH;
+
+	log_dbg(cd, "Setting PBKDF2 type key digest %d.", digest);
 
 	r = crypt_random_get(cd, salt, LUKS_SALTSIZE, CRYPT_RND_SALT);
 	if (r < 0)
@@ -124,8 +130,8 @@ static int PBKDF2_digest_store(struct crypt_device *cd,
 	}
 
 	hmac_size = crypt_hmac_size(pbkdf.hash);
-	if (hmac_size < 0)
-		return hmac_size;
+	if (hmac_size < 0 || hmac_size > (int)sizeof(digest_raw))
+		return -EINVAL;
 
 	r = crypt_pbkdf(CRYPT_KDF_PBKDF2, pbkdf.hash, volume_key, volume_key_len,
 			salt, LUKS_SALTSIZE, digest_raw, hmac_size,
@@ -147,28 +153,26 @@ static int PBKDF2_digest_store(struct crypt_device *cd,
 	json_object_object_add(jobj_digest, "hash", json_object_new_string(pbkdf.hash));
 	json_object_object_add(jobj_digest, "iterations", json_object_new_int(pbkdf.iterations));
 
-	base64_encode_alloc(salt, LUKS_SALTSIZE, &base64_str);
-	if (!base64_str) {
+	r = crypt_base64_encode(&base64_str, NULL, salt, LUKS_SALTSIZE);
+	if (r < 0) {
 		json_object_put(jobj_digest);
-		return -ENOMEM;
+		return r;
 	}
 	json_object_object_add(jobj_digest, "salt", json_object_new_string(base64_str));
 	free(base64_str);
 
-	base64_encode_alloc(digest_raw, hmac_size, &base64_str);
-	if (!base64_str) {
+	r = crypt_base64_encode(&base64_str, NULL, digest_raw, hmac_size);
+	if (r < 0) {
 		json_object_put(jobj_digest);
-		return -ENOMEM;
+		return r;
 	}
 	json_object_object_add(jobj_digest, "digest", json_object_new_string(base64_str));
 	free(base64_str);
 
-	if (jobj_digests) {
-		snprintf(num, sizeof(num), "%d", digest);
-		json_object_object_add(jobj_digests, num, jobj_digest);
-	}
+	if (jobj_digests)
+		json_object_object_add_by_uint(jobj_digests, digest, jobj_digest);
 
-	JSON_DBG(jobj_digest, "Digest JSON");
+	JSON_DBG(cd, jobj_digest, "Digest JSON:");
 	return 0;
 }
 
